from services.service import Service
from typing import List
import requests
from requests import Response
from bs4 import BeautifulSoup, Tag
from services.settings import HTTP_SUCCESS_CODE, Log, BASE_DIR_DOWNLOAD, update_download_history, TRADINGVIEW_CSV_ORIGIN_FILENAME

class TradingViewService(Service):

    _SYMBOL = ":stock:"
    _URL = f"https://br.tradingview.com/symbols/BMFBOVESPA-{_SYMBOL}/"
    _FILE_PATH = f"{BASE_DIR_DOWNLOAD}/{TRADINGVIEW_CSV_ORIGIN_FILENAME}"

    def config_step(self):
        Log.log("Start")
        HEADER = "STOCK;NAME;DEPARTMENT;IMAGE\n"
        Log.log("Open file")
        self.file = open(self._FILE_PATH, "a", encoding="utf-8")
        self.file.write(HEADER)
        self.lines = []

    def make_request(self):
        url = self._URL.replace(self._SYMBOL, self.stock)
        response: Response = requests.get(url)

        if response.status_code == HTTP_SUCCESS_CODE:
            html = response.text
            self.page = BeautifulSoup(html, 'html.parser')
            self.execute_next_step = True
        else:
            Log.log(f"Failed request with status code {response.status_code} when call for {self.stock}")
            self.execute_next_step = False

    def read_page_and_get_data(self):
            if not self.execute_next_step:
                Log.log("Skip step")
                return 

            img_tag = self.page.select_one('img[class*="logo-"]')            
            image = img_tag["src"]

            name_tag = self.page.find("h1", class_="apply-overflow-tooltip title-HDE_EEoW")
            name = name_tag.get_text(strip=True)

            prefix = "/markets/stocks-brazil/sectorandindustry-sector/"
            department_tag = self.page.select_one(f'a[href^="{prefix}"]')
            
            if department_tag == None:
                department = ""
            else:
                department = department_tag.get_text(strip=True)

            self.lines.append(f"{self.stock};{name};{department};{image}\n")

    def transform_data_into_csv(self):
        Log.log("Writing lines")
        self.file.writelines(self.lines)
        Log.log("Close file")
        self.file.close()
        update_download_history(TRADINGVIEW_CSV_ORIGIN_FILENAME)

    def run(self, stocks: List[str]):
        self.config_step()
        for stock in stocks:
            self.stock = stock
            self.make_request()
            self.read_page_and_get_data()
        self.transform_data_into_csv()


"""
Financial Scraper - A library for scraping Brazilian market data
"""

from .providers.status_invest import StatusInvestProvider
from .providers.fundamentus import FundamentusProvider
from .providers.investor_ten import InvestorTenProvider
from .providers.trading_view import TradingViewProvider
from .providers.market_data import MarketDataService

__version__ = "1.0.0"
__all__ = ['StatusInvestProvider', 'FundamentusProvider',
           'InvestorTenProvider', 'TradingViewProvider', 'MarketDataService']



from selenium.webdriver.chrome.options import Options


class Selenium():

    @staticmethod
    def get_options(download_path: str, show_browser: bool = False) -> Options:
        options = Options()
        if not show_browser:
            options.add_argument("--headless=new")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-dev-shm-usage")
        options.add_argument("--disable-gpu")
        options.add_argument("--window-size=1920,1080")
        options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36")
        prefs = {
            "download.default_directory": download_path,
            "download.prompt_for_download": False,
            "download.directory_upgrade": True,
            "safebrowsing.enabled": True,
            "profile.default_content_settings.popups": 0,
            "profile.content_settings.exceptions.automatic_downloads.*.setting": 1,
            "safebrowsing.disable_download_protection": True,
            "browser.download.folderList": 2,
            "browser.helperApps.neverAsk.saveToDisk": "application/csv,text/csv,application/vnd.ms-excel"
        }
        options.add_experimental_option('prefs', prefs)
        
        # Disable the "Save As" dialog and enable downloads in headless mode
        options.add_experimental_option("excludeSwitches", ["enable-automation"])
        options.add_experimental_option('useAutomationExtension', False)
        return options



from abc import ABC, abstractmethod

class Service(ABC):

    def __init__(self):
        super().__init__()

    @abstractmethod
    def config_step(self):
        pass

    @abstractmethod
    def make_request(self):
        pass

    @abstractmethod
    def read_page_and_get_data(self):
        pass

    @abstractmethod
    def transform_data_into_csv(self):
        pass

    def run(self):
        self.config_step()
        self.make_request()
        self.read_page_and_get_data()
        self.transform_data_into_csv()


from services.service import Service
from services.settings import Log
from services.settings import Log, Selenium, BASE_DIR_DOWNLOAD, check_if_file_was_downloaded, update_download_history, MARKETDATA_CSV_ORIGIN_FILENAME, BASE_DIR_DATA_FUNDS_PROFITS
from selenium import webdriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
import csv
from typing import List

class InvestorTenService(Service):

    _URL = "https://investidor10.com.br/fiis/dividendos/:year:/:month:/"
    _MONTHS = ["janeiro", "fevereiro", "marco", "abril", "maio", "junho", "julho", "agosto", "setembro", "outubro", "novembro", "dezembro"]

    def config_step(self):
        Log.log(f"Start")
        options = Selenium.get_options()
        self.driver = webdriver.Chrome(options=options)
    
    def make_request(self):
        Log.log(f"Start for month {self.month}")
        url = self._URL.replace(":year:", self.year).replace(":month:", self.month)
        Log.log(f"Url: {url}")

        try:
            self.driver.get(url)
            table = WebDriverWait(self.driver, 10).until(
                EC.presence_of_element_located((By.CSS_SELECTOR, "table.min-w-full.bg-white.md\\:shadow.border-collapse.border-spacing-0"))
            )

            self.rows = table.find_elements(By.CSS_SELECTOR, "tbody tr")
        except Exception as e:
            Log.log_error("Error in request", e)
            self.driver.quit()
    
    def read_page_and_get_data(self):
        Log.log(f"Start for month {self.month}")
        for row in self.rows:
            cols = [col.text.strip() for col in row.find_elements(By.TAG_NAME, "td")]
            if cols:
                cleaned = self.clean_data(cols)
                self.result.append(cleaned)
        
        Log.log(f"Number of lines: {len(self.result)}")
        
        self.driver.quit()


    def transform_data_into_csv(self):
        Log.log(f"Start")
        headers = ["FII", "Data Com", "Data Pagamento", "Tipo", "Valor"]
        filename = f"funds-profits-{self.year}.csv"
        path = f"{BASE_DIR_DATA_FUNDS_PROFITS}/{filename}"
        with open(path, "a", newline="", encoding="utf-8") as f:
            writer = csv.writer(f, delimiter=';')
            writer.writerow(headers)
            writer.writerows(self.result)
            Log.log(f"Create file {path}")
        
        update_download_history(filename)
    
    def run(self, year: str):
        self.year = year
        self.result = []
        for month in self._MONTHS:
            self.month = month 
            self.config_step()
            self.make_request()
            self.read_page_and_get_data()
        self.transform_data_into_csv()
        

    def clean_data(self, row: List[str]) -> List[str]:
        # Clean column FII
        try:
            row[0] = row[0].split('\n')[0]
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row} in column FII", e)
            return row
        
        # Clean column Data com
        try:
            row[1] = row[1].replace('/', '-')
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row[0]} in column Data com. Row: {row}", e)
        
        # Clean column Pagamento com
        try:
            row[2] = row[2].replace('/', '-')
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row[0]} in column Pagamento com. Row: {row}", e)


        # Clean column Valor
        try:
            row[4] = row[4].split(' ')[1].replace(',', '.')
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row[0]} in column . Row: {row}", e)
        
        return row



import csv
import os
import requests
from typing import List
from bs4 import BeautifulSoup
from financial_scraper.config.utils import Log


class InvestorTenProvider():
    """
    A provider for scraping Real Estate Investment Trust (FIIS) dividend data from Investidor10 website.
    
    This class retrieves information about dividends paid by Brazilian REITs (FIIs) 
    for specified years and months. The data is collected from the Investidor10 
    website and saved into a CSV file.
    
    Attributes:
        download_path (str): Directory path where the CSV file will be saved.
        filename (str, optional): Custom filename for the output CSV file.
                                 If not provided, defaults to 'funds-{year}.csv'.
    """

    _URL = "https://investidor10.com.br/fiis/dividendos/:year:/:month:/"
    _MONTHS = ["janeiro", "fevereiro", "marco", "abril", "maio", "junho",
               "julho", "agosto", "setembro", "outubro", "novembro", "dezembro"]
    _FILENAME = "funds-:year:.csv"
    _HEADERS = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
        'Accept-Language': 'pt-BR,pt;q=0.9,en-US;q=0.8,en;q=0.7',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
        'Referer': 'https://investidor10.com.br',
        'Connection': 'keep-alive'
    }

    def __init__(self, download_path: str, filename: str = None):
        super().__init__()
        self.download_path = download_path
        self.filename = filename
        os.makedirs(self.download_path, exist_ok=True)

    def _make_request(self):
        Log.log(f"Start for month {self.month}")
        url = self._URL.replace(":year:", self.year).replace(":month:", self.month)
        Log.log(f"Url: {url}")

        try:
            response = requests.get(url, headers=self._HEADERS)
            response.raise_for_status()
            self.response = response.text
        except Exception as e:
            Log.log_error(f"Error making request for {self.month} {self.year}", e)
            raise e

        try:
            soup = BeautifulSoup(self.response, 'html.parser')
            self.table = soup.select_one("table.min-w-full.bg-white.md\\:shadow.border-collapse.border-spacing-0")
            if not self.table:
                Log.log(f"No dividend table found for {self.month} {self.year}")
                return
        except Exception as e:
            Log.log_error(f"Error parsing HTML for {self.month} {self.year}", e)
            raise e

    def _read_page_and_get_data(self):
        rows = self.table.select("tbody tr")
        Log.log(f"Found {len(rows)} rows for {self.month}")
        for row in rows:
            cols = row.select("td")
            if not cols:
                continue

            ticker = cols[0].select_one(".ticker-name").get_text(strip=True)
            vector_cols = [col.get_text(strip=True) for col in cols]
            if vector_cols:
                vector_cols[0] = ticker
                cleaned = self._clean_data(vector_cols)
                self.result.append(cleaned)

    def _transform_data_into_csv(self):
        Log.log("Start")
        headers = ["FII", "Data Com", "Data Pagamento", "Tipo", "Valor"]
        path = f"{self.download_path}/{self._FILENAME.replace(':year:', self.year)}"
        if self.filename:
            path = f"{self.download_path}/{self.filename}"
        with open(path, "a", newline="", encoding="utf-8") as f:
            writer = csv.writer(f, delimiter=';')
            writer.writerow(headers)
            writer.writerows(self.result)
            Log.log(f"Create file {path}")

    def run(self, year: str):
        """
        Run the scraping process for a specific year.
        
        This is the main method to execute the scraping process. It iterates through
        all months of the specified year, collects dividend data for each month,
        and saves the results to a CSV file.
        
        Args:
            year (str): The year to collect dividend data for (e.g., "2023").
        """
        self.year = year
        self.result = []
        for month in self._MONTHS:
            self.month = month
            try:
                self._make_request()
            except:
                continue
            self._read_page_and_get_data()
        self._transform_data_into_csv()

    def _clean_data(self, row: List[str]) -> List[str]:
        # Clean column FII
        try:
            row[0] = row[0].split('\n')[0]
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row} in column FII", e)
            return row

        # Clean column Data com
        try:
            row[1] = row[1].replace('/', '-')
            row[1] = row[1].replace('Data Com', '').strip()
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row[0]} in column Data com. Row: {row}", e)

        # Clean column Pagamento com
        try:
            row[2] = row[2].replace('/', '-')
            row[2] = row[2].replace('Pgto', '').strip()
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row[0]} in column Pagamento com. Row: {row}", e)

        try:
            value_text = row[4]
            if "R$" in value_text:
                value_text = value_text.split("R$")[1].strip()
            row[4] = value_text.replace(',', '.')
        except Exception as e:
            Log.log_error(f"Unable to clean data of {row[0]} in column Valor. Row: {row}", e)

        return row



from services.service import Service
from services.settings import Log, Selenium, BASE_DIR_DOWNLOAD, check_if_file_was_downloaded, update_download_history, MARKETDATA_CSV_ORIGIN_FILENAME
from selenium import webdriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
import os, time

class MarketDataService(Service):

    _URL = "https://www.dadosdemercado.com.br/acoes"

    def config_step(self):
        Log.log("Start")
        options = Selenium.get_options()
        self.driver = webdriver.Chrome(options=options)
    
    def make_request(self):
        Log.log("Start")
        self.driver.get(self._URL)

        try:
            Log.log("Get download button")
            download_button = WebDriverWait(self.driver, 10).until(
                EC.element_to_be_clickable((By.ID, "download-csv"))
            )

            Log.log("Click download button")
            self.driver.execute_script("arguments[0].click();", download_button)

            Log.log(f"Save file in {BASE_DIR_DOWNLOAD}")

            timeout = 30
            is_file_downloaded = check_if_file_was_downloaded(MARKETDATA_CSV_ORIGIN_FILENAME, timeout)
            if is_file_downloaded:
                Log.log("Download completed!")
                update_download_history(MARKETDATA_CSV_ORIGIN_FILENAME)

            else:
                Log.log("Erro to found .csv into downloads folder!")

        except Exception as e:
            Log.log_error("Error when try to download csv", e)
        finally:
            self.driver.quit()

    def read_page_and_get_data(self):
        Log.log("Skip read page and get data step")

    def transform_data_into_csv(self):
        Log.log("Skip transform data into csv")


import inspect
import time
import os


class Log():
    def _get_caller_name() -> str:
        caller_frame = inspect.stack()[2].frame
        caller_function = inspect.stack()[2].function
        caller_self = caller_frame.f_locals.get('self', None)
        if caller_self:
            return f"[{caller_self.__class__.__name__}] {Log._remove_leading_underscore(caller_function).upper()}."
        else:
            return f"[No Class] {caller_function.upper()}"

    def log(msg: str):
        caller = Log._get_caller_name()
        print(f'{caller}: {msg}')

    def log_error(msg: str, error: Exception):
        caller = Log._get_caller_name()
        print(f'{caller}: {msg}')
        print(f'Root cause: {error}')

    def _remove_leading_underscore(text: str) -> str:
        if text.startswith("_"):
            return text[1:]
        return text

def check_if_file_was_downloaded(filename: str, timeout: int, download_path: str) -> bool:
    found = False
    for _ in range(timeout):
        files = [f for f in os.listdir(download_path) if f.endswith(filename)]
        if files:
            found = True
            break
        time.sleep(1)

    return found



from selenium import webdriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from financial_scraper.config.selenium import Selenium
from financial_scraper.config.utils import Log, check_if_file_was_downloaded
import os


class MarketDataService():
    """
    A service for downloading comprehensive Brazilian stock market data from dadosdemercado.com.br.
    
    This class retrieves a complete list of stocks listed on B3 (Brazil's stock exchange)
    with their respective information. The data is downloaded as a CSV file and can be 
    saved with a custom filename.
    
    Attributes:
        download_path (str): Directory path where the CSV file will be saved.
        filename (str, optional): Custom filename for the output CSV file.
                                 If not provided, the original filename is used.
        show_browser (bool): Whether to show the browser window during scraping.
    """

    _URL = "https://www.dadosdemercado.com.br/acoes"
    _MARKETDATA_CSV_ORIGIN_FILENAME = "acoes-listadas-b3.csv"

    def __init__(self, download_path: str, filename: str = None, show_browser: bool = False):
        """
        Initialize the MarketDataService.

        Args:
            download_path (str): Directory path where downloaded files will be saved.
            filename (str, optional): Custom filename for the downloaded CSV file.
                If None, a default filename will be used.
            show_browser (bool, optional): Whether to show the browser window during execution.
                Defaults to False (headless mode).
        """
        self.filename = filename
        self.download_path = download_path
        self.show_browser = show_browser

    def config_step(self):
        Log.log("Start")
        options = Selenium.get_options(download_path=self.download_path, show_browser=self.show_browser)
        self.driver = webdriver.Chrome(options=options)

    def make_request(self):
        Log.log("Start")
        self.driver.get(self._URL)

        try:
            Log.log("Get download button")
            download_button = WebDriverWait(self.driver, 10).until(
                EC.element_to_be_clickable((By.ID, "download-csv"))
            )

            Log.log("Click download button")
            self.driver.execute_script("arguments[0].click();", download_button)

            Log.log(f"Save file in {self.download_path}")

            timeout = 30
            is_file_downloaded = check_if_file_was_downloaded(
                self._MARKETDATA_CSV_ORIGIN_FILENAME, timeout, self.download_path)
            if is_file_downloaded:
                Log.log("Download completed!")
            else:
                Log.log("Erro to found .csv into downloads folder!")
        except Exception as e:
            Log.log_error("Error when try to download csv", e)
        finally:
            self.driver.quit()

    def transform_data_into_csv(self):
        Log.log("Start")
        origin_file_path = os.path.join(self.download_path, self._MARKETDATA_CSV_ORIGIN_FILENAME)
        if self.filename:
            os.rename(origin_file_path, os.path.join(self.download_path, self.filename))

    def run(self):
        """
        Run the complete process to download stock market data.
        
        This is the main method that executes the entire workflow:
        1. Configure the Selenium WebDriver
        2. Make a request to the website and download the CSV file
        3. Rename the file if a custom filename was provided
        
        Example:
            >>> service = MarketDataService(download_path="./data")
            >>> service.run()
        """
        self.config_step()
        self.make_request()
        self.transform_data_into_csv()



import pandas as pd
import requests
from bs4 import BeautifulSoup
from datetime import datetime
from financial_scraper.config.utils import Log


class FundamentusProvider():
    """
    Provider for scraping fundamental stock data from Fundamentus website.

    This class uses the requests library to fetch HTML content from Fundamentus
    and BeautifulSoup to parse the data. It extracts fundamental indicators for
    Brazilian stocks and saves them as a CSV file.

    Attributes:
        download_path (str): Directory where the CSV file will be saved.
    """
    _URL = "https://fundamentus.com.br/resultado.php"
    _HEADERS = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36' +
        '(KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
    }

    def __init__(self, download_path: str, filename: str = None):
        """
        Initialize the FundamentusProvider.

        Args:
            download_path (str): Directory path where the CSV file will be saved.
        """
        super().__init__()
        self._data = []
        self.filename = filename
        self.download_path = download_path

    def _make_request(self):
        try:
            Log.log(f"Making request to {self._URL}")
            response = requests.get(self._URL, headers=self._HEADERS)
            response.raise_for_status()
            self.html_content = response.content
            Log.log("Request successful")
        except Exception as e:
            Log.log_error("Error making request to Fundamentus", e)
            raise e

    def _read_page_and_get_data(self):
        try:
            Log.log("Parsing HTML content")
            soup = BeautifulSoup(self.html_content, 'html.parser')
            table = soup.find('table')

            headers = []
            for th in table.find_all('th'):
                headers.append(th.text.strip())

            for tr in table.find_all('tr')[1:]:
                row = []
                for td in tr.find_all('td'):
                    row.append(td.text.strip())
                if row:
                    self._data.append(row)

            Log.log(f"Successfully parsed {len(self._data)} rows of data")

            self.df = pd.DataFrame(self._data, columns=headers)

        except Exception as e:
            Log.log_error("Error parsing HTML content", e)
            raise e

    def _transform_data_into_csv(self):
        try:
            current_date = datetime.now().strftime("%d-%m-%Y")
            filename = f"fundamentus-{current_date}.csv"
            if self.filename:
                filename = self.filename
            filepath = f"{self.download_path}/{filename}"

            Log.log(f"Saving data to {filepath}")
            self.df.to_csv(filepath, index=False, sep=';')
            Log.log("Data successfully saved to CSV")

        except Exception as e:
            Log.log_error("Error saving data to CSV", e)
            raise e

    def run(self):
        """
        Execute the complete scraping process for Fundamentus data.

        This is the main public method to run the scraper. It performs all steps:
        configuration, making the HTTP request, parsing the HTML data, and saving
        the results to a CSV file.

        Returns:
            None: The results are saved as a CSV file in the download_path.
        """
        self._make_request()
        self._read_page_and_get_data()
        self._transform_data_into_csv()



import time
import csv
import os
from datetime import datetime
from bs4 import BeautifulSoup
from services.settings import Log, BASE_DIR, Selenium, update_download_history, BASE_DIR_DATA_FUNDS
from selenium import webdriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.chrome.options import Options
from services.service import Service

class FundsExplorerService(Service):

    _FUNDS_EXPLORER_URL: str = "https://www.fundsexplorer.com.br/ranking"
    _TIME_TO_SLEEP: int = 10

    def config_step(self):
        Log.log("Start")
        options = Selenium.get_options()
        self.driver = webdriver.Chrome(options=options)

    def make_request(self):
        Log.log("Start")
        try:
            self.driver.get(self._FUNDS_EXPLORER_URL)
            Log.log(f"Accessing {self._FUNDS_EXPLORER_URL}")
            time.sleep(self._TIME_TO_SLEEP)
            self.source_code = self.driver.page_source
        except Exception as e:
            Log.log_error("Error when extract data", e)
        finally:
            self.driver.quit()
    
    def read_page_and_get_data(self):
        Log.log("Start")
        self.rows = []

        soup = BeautifulSoup(self.source_code, "html.parser")
        table = soup.select_one("table")
        if not table:
            return None

        Log.log("Get header from table")
        self.heads = [th.get_text(strip=True) for th in table.select("thead th")]
        if not self.heads:
            return None
        Log.log("Load table header")
        
        Log.log("Get information from table")
        body_rows = table.select("tbody tr")
        for tr in body_rows:
            cells = [td.get_text(strip=True) for td in tr.select("td,th")]
            self.rows.append(cells)
        Log.log(f"Load {len(self.rows)} itens")

    def transform_data_into_csv(self):
        Log.log("Start")
        today = datetime.today().strftime("%d-%m-%Y")
        filename = f"funds-{today}.csv"
        path = f"{BASE_DIR_DATA_FUNDS}/{filename}"
        Log.log(f"Get path: {path}")

        with open(path, "w", encoding="utf-8") as f:
            writer = csv.writer(f)
            Log.log("Write header into file")
            writer.writerow(self.heads)
            Log.log("Write rows into file")
            writer.writerows(self.rows)
        
        update_download_history(filename)


# Examples of using the market_scraper library
from financial_scraper import StatusInvestProvider, FundamentusProvider, InvestorTenProvider, TradingViewProvider, MarketDataService
import os

BASE_DIR = os.path.dirname(os.path.abspath(__file__))


def status_invest_example():
    # Initialize the service with Status Invest provider
    service = StatusInvestProvider(
        download_path=BASE_DIR,
        filename="status_invest_stocks.csv",
        show_browser=True
    )

    # Fetch and save data
    service.run(sector=StatusInvestProvider.Sector.FINANCIAL_AND_OTHERS)


def fundamentus_example():
    # Initialize the service with Fundamentus provider
    service = FundamentusProvider(
        download_path=BASE_DIR,
        filename="fundamentus_stocks.csv"
    )

    # Fetch and save data
    service.run()


def investor_ten_example():
    """
    Example showing how to use the InvestorTenProvider to scrape REIT (FII) dividend data
    from Investidor10 website for a specific year.
    """
    # Initialize the provider with default filename (funds-{year}.csv)
    service = InvestorTenProvider(
        download_path=BASE_DIR
    )

    # Fetch and save data for 2023
    # This will create a file named 'funds-2023.csv'
    service.run("2023")

    # Example with custom filename
    service_custom = InvestorTenProvider(
        download_path=BASE_DIR,
        filename="fiis-dividends-2024.csv"
    )

    # Fetch and save data for 2024 with custom filename
    service_custom.run("2024")


def trading_view_example():
    # Get stocks list (in this example we use the csv get by status_invest_example)
    stocks_file = os.path.join(BASE_DIR, "stocks_for_trading_view_example.csv")
    with open(stocks_file, "r") as f:
        stocks = [line.split(";")[0] for line in f.readlines()[1:]]  # Skip header
    stocks = stocks[:10]

    # Initialize the service with Trading View provider
    service = TradingViewProvider(
        download_path=BASE_DIR,
        filename="trading_view_stocks.csv"
    )
    service.run(stocks=stocks)


def market_data_example():
    # Initialize the service with Market Data provider
    service = MarketDataService(
        download_path=BASE_DIR,
        filename="market_data_stocks.csv",
        show_browser=True
    )

    # Fetch and save data
    service.run()


if __name__ == "__main__":
    status_invest_example()
    fundamentus_example()
    investor_ten_example()



from financial_scraper.config.selenium import Selenium
from financial_scraper.config.utils import Log, check_if_file_was_downloaded
from selenium import webdriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
import os
from enum import Enum


class StatusInvestProvider():
    """
    Provider for scraping stock data from Status Invest website.
    
    This class uses Selenium WebDriver to navigate to Status Invest's advanced search page,
    filter stocks by sector (if specified), and download stock data in CSV format.
    
    Attributes:
        download_path (str): Directory where the downloaded CSV will be saved.
        filename (str, optional): Custom filename for the downloaded CSV.
        show_browser (bool): Whether to show the browser window during scraping.
        sector (Sector): Sector filter to apply during the search.
    """

    class Sector(Enum):
        """
        Enumeration of stock market sectors available on Status Invest.
        
        Each enum value is a tuple containing:
        - First element: URL/filename-friendly sector name
        - Second element: Display name as shown on Status Invest website
        """
        CYCLIC_CONSUMPTION = ("cyclic-consumption", "Consumo Cíclico")
        NON_CYCLIC_CONSUMPTION = ("non-cyclic-consumption", "Consumo não Cíclico")
        PUBLIC_UTILITIES = ("public-utilities", "Utilidade Pública")
        INDUSTRIAL_GOODS = ("industrial-goods", "Bens Industriais")
        BASIC_MATERIALS = ("basic-materials", "Materiais Básicos")
        FINANCIAL_AND_OTHERS = ("financial-and-others", "Financeiro e Outros")
        INFORMATION_TECHNOLOGY = ("information-technology", "Tecnologia da Informação")
        HEALTHCARE = ("healthcare", "Saúde")
        OIL_GAS_AND_BIOFUELS = ("oil-gas-and-biofuels", "Petróleo. Gás e Biocombustíveis")
        COMMUNICATIONS = ("communications", "Comunicações")
        UNDEFINED = ("undefined", "Indefinido")

    _SEARCH_BUTTON_DATA_TOOLTIP = "Clique para fazer a busca com base nos valores informados"
    _URL = "https://statusinvest.com.br/acoes/busca-avancada"
    _STATUSINVEST_CSV_ORIGIN_FILENAME = "statusinvest-busca-avancada.csv"
    _STATUSINVEST_CSV_NEW_STOCKS_FILENAME = "statusinvest:sector:.csv"
    _NO_SECTOR = ""

    def __init__(self, download_path: str, filename: str = None, show_browser: bool = False):
        """
        Initialize the StatusInvestProvider.
        
        Args:
            download_path (str): Directory path where downloaded files will be saved.
            filename (str, optional): Custom filename for the downloaded CSV file.
                If None, a default filename based on sector will be used.
            show_browser (bool, optional): Whether to show the browser window during execution.
                Defaults to False (headless mode).
        """
        super().__init__()
        self.filename = filename
        self.download_path = download_path
        self.show_browser = show_browser

    def _config_step(self):
        Log.log("Start")
        options = Selenium.get_options(self.download_path, self.show_browser)
        self.driver = webdriver.Chrome(options=options)

    def _make_request(self):
        Log.log("Start")
        self.driver.get(self._URL)

        try:
            if (self.sector != StatusInvestProvider.Sector.UNDEFINED):
                self._select_sector()

            Log.log("Get search button")
            search_button = WebDriverWait(self.driver, 10).until(
                EC.element_to_be_clickable((By.XPATH, f"//button[@data-tooltip='{self._SEARCH_BUTTON_DATA_TOOLTIP}']"))
            )

            Log.log("Click search button")
            self.driver.execute_script("arguments[0].click();", search_button)

            Log.log("Get download button")
            download_button = WebDriverWait(self.driver, 10).until(
                EC.element_to_be_clickable((By.CSS_SELECTOR, "a.btn-download"))
            )

            Log.log("Click download button")
            self.driver.execute_script("arguments[0].click();", download_button)

            Log.log(f"Save file in {self.download_path}")

            timeout = 30
            is_file_downloaded = check_if_file_was_downloaded(
                self._STATUSINVEST_CSV_ORIGIN_FILENAME, timeout, self.download_path)
            if is_file_downloaded:
                Log.log("Download completed!")
                self._rename_file()
            else:
                Log.log(f"Erro to found .csv into {self.download_path}!")

        except Exception as e:
            Log.log_error("Error when try to download csv", e)
        finally:
            self.driver.quit()

    def _rename_file(self):
        # TODO: Make this more readeble
        sector_string = self._NO_SECTOR
        if (self.sector != StatusInvestProvider.Sector.UNDEFINED):
            sector_string = f"-{self.sector.value[0]}"

        filename = self._STATUSINVEST_CSV_NEW_STOCKS_FILENAME.replace(":sector:", sector_string)

        if self.filename is not None:
            filename = self.filename

        new_path = f"{self.download_path}/{filename}"
        old_path = f"{self.download_path}/{self._STATUSINVEST_CSV_ORIGIN_FILENAME}"
        os.rename(old_path, new_path)

    def _select_sector(self):
        Log.log(f"Select sector {self.sector}")
        Log.log("Search for dropdown-item Sectors")
        span_element = WebDriverWait(self.driver, 10).until(
            EC.presence_of_element_located(
                (By.XPATH, "//span[text()='-- Todos setores --']")
            )
        )
        dropdown_item = span_element.find_element(By.XPATH, "./ancestor::div[@class='select-wrapper']/input")

        Log.log("Click to open sector dropdown")
        dropdown_item.click()

        Log.log("Wait to the options")
        option = WebDriverWait(self.driver, 10).until(
            EC.visibility_of_element_located(
                (By.XPATH,
                 f"//ul[contains(@class,'select-dropdown')]/li/span[normalize-space()='{self.sector.value[1]}']")
            )
        )
        Log.log(f"Click to {self.sector}")
        option.click()

    def run(self, sector: Sector = Sector.UNDEFINED):
        """
        Execute the complete scraping process to fetch stock data.
        
        This is the main public method to run the scraper. It configures the WebDriver,
        navigates to the website, applies filters, downloads the data, and processes it.
        
        Args:
            sector (Sector, optional): Specific sector to filter stocks by.
                Defaults to Sector.UNDEFINED (no sector filter).
                
        Returns:
            None: The results are saved as a CSV file in the download_path.
        """
        self.sector = sector
        self._config_step()
        self._make_request()



from typing import List
import requests
from requests import Response
from bs4 import BeautifulSoup
from financial_scraper.config.utils import Log


class TradingViewProvider():
    """
    A provider for scraping stock information from TradingView website.
    
    This class retrieves basic information about stocks from the Brazilian stock market
    (B3/BMFBOVESPA) including stock name, department/sector, and logo image URL.
    The data is collected from the TradingView website and saved into a CSV file.
    
    Attributes:
        download_path (str): Directory path where the CSV file will be saved.
        filename (str, optional): Custom filename for the output CSV file.
                                 If not provided, defaults to 'trading_view.csv'.
    """

    _SYMBOL = ":stock:"
    _URL = f"https://br.tradingview.com/symbols/BMFBOVESPA-{_SYMBOL}/"
    # _URL_STATISTICS = f"https://br.tradingview.com/symbols/BMFBOVESPA-{_SYMBOL}/financials-statistics-and-ratios/"
    # _URL_DEMOSTRATIONS = f"https://br.tradingview.com/symbols/BMFBOVESPA-{_SYMBOL}/financials-income-statement/"
    _DEFAULT_FILENAME = "trading_view.csv"
    # - SEARCH_STRING - HEADER

    def __init__(self, download_path: str, filename: str = None):
        """
        Initialize the TradingViewProvider with a download path and optional filename.
        
        Args:
            download_path (str): Directory path where the CSV file will be saved.
            filename (str, optional): Custom filename for the output CSV file.
                                      If not provided, defaults to 'trading_view.csv'.
        """
        self.download_path = download_path
        self.filename = filename if filename else self._DEFAULT_FILENAME

    def _config_step(self):
        Log.log("Start")
        self.HEADER = "STOCK;NAME;DEPARTMENT;IMAGE\n"
        self.lines = []

    def _make_request(self):
        url = self._URL.replace(self._SYMBOL, self.stock)
        response: Response = requests.get(url)
        Log.log(f"Making request for {self.stock}")

        if response.status_code == 200:
            html = response.text
            self.page = BeautifulSoup(html, 'html.parser')
            self.execute_next_step = True
        else:
            Log.log(f"Failed request with status code {response.status_code} when call for {self.stock}")
            self.execute_next_step = False

    def _read_page_and_get_data(self):
        if not self.execute_next_step:
            Log.log("Skip step")
            return

        Log.log(f"Reading page and getting data for {self.stock}")

        img_tag = self.page.select_one('img[class*="logo-"]')
        image = ""
        if img_tag is not None:
            image = img_tag["src"]

        name_tag = self.page.find("h1", class_="apply-overflow-tooltip title-HDE_EEoW")
        name = ""
        if name_tag is not None:
            name = name_tag.get_text(strip=True)

        prefix = "/markets/stocks-brazil/sectorandindustry-sector/"
        department_tag = self.page.select_one(f'a[href^="{prefix}"]')

        if department_tag is None:
            department = ""
        else:
            department = department_tag.get_text(strip=True)

        self.lines.append(f"{self.stock};{name};{department};{image}\n")

    def _transform_data_into_csv(self):
        if not self.lines:
            Log.log("No data to write")
            return

        file_path = f"{self.download_path}/{self.filename}"
        Log.log(f"Writing data to CSV at {file_path}")
        with open(file_path, "w", encoding="utf-8") as f:
            f.write(self.HEADER)
            f.writelines(self.lines)

    def run(self, stocks: List[str]):
        """
        Run the scraping process for a list of stock tickers.
        
        This is the main method to execute the scraping process. It iterates through
        the provided list of stock tickers, collects information for each stock,
        and saves the results to a CSV file.
        
        Args:
            stocks (List[str]): A list of stock ticker symbols to collect data for.
                               These should be valid B3/BMFBOVESPA stock symbols.
                               
        Example:
            >>> provider = TradingViewProvider(download_path="./data")
            >>> provider.run(stocks=["PETR4", "VALE3", "ITUB4"])
        """
        self._config_step()
        for stock in stocks:
            self.stock = stock
            self._make_request()
            self._read_page_and_get_data()
        self._transform_data_into_csv()



import os, time
from datetime import datetime
from selenium.webdriver.chrome.options import Options
import inspect
from pathlib import Path

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
BASE_DIR_DOWNLOAD = os.path.join(os.path.dirname(BASE_DIR), "downloads")
BASE_DIR_DATA = os.path.join(os.path.dirname(BASE_DIR), "data/stocks")
BASE_DIR_DATA_FUNDS = os.path.join(os.path.dirname(BASE_DIR), "data/funds")
BASE_DIR_DATA_FUNDS_PROFITS = os.path.join(os.path.dirname(BASE_DIR), "data/funds_profits")
HTTP_SUCCESS_CODE = 200

MARKETDATA_CSV_ORIGIN_FILENAME = "acoes-listadas-b3.csv"

TRADINGVIEW_CSV_ORIGIN_FILENAME = "trading-view-stocks.csv"


class Log():
    def _get_caller_name()-> str:
        caller_frame = inspect.stack()[2].frame
        caller_function = inspect.stack()[2].function
        caller_self = caller_frame.f_locals.get('self', None)
        if caller_self:
            return f"[{caller_self.__class__.__name__}] {caller_function.upper()}"
        else:
            return f"[No Class] {caller_function.upper()}"

    def log(msg: str):
        caller = Log._get_caller_name()
        print(f'{caller}: {msg}')
    
    def log_error(msg: str, error: Exception):
        caller = Log._get_caller_name()
        print(f'{caller}: {msg}')
        print(f'Root cause: {error}')


class Selenium():

    @staticmethod
    def get_options(download_path: str) -> Options:
        options = Options()
        options.add_argument("--headless=new")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-dev-shm-usage")
        options.add_argument("--disable-gpu")
        options.add_argument("--window-size=1920,1080")
        options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36")
        prefs = {
            "download.default_directory": download_path,
            "download.prompt_for_download": False,
            "download.directory_upgrade": True,
            "safebrowsing.enabled": True,
        }
        options.add_experimental_option('prefs', prefs)
        return options


def check_if_file_was_downloaded(filename: str, timeout: int, download_path: str) -> bool:
    found = False
    for _ in range(timeout):
        files = [f for f in os.listdir(download_path) if f.endswith(filename)]
        if files:
            found = True
            break
        time.sleep(1)

    return found


def update_download_history(filename: str):
    FILENAME = f"{BASE_DIR_DOWNLOAD}/download_history.csv"
    HEADER = "NAME;DATE;TIME\n"
    file_path = Path(FILENAME)
    edit_mode = 'a'
    put_header = False
    if not file_path.exists():
        edit_mode = 'w'
        put_header = True

    with open(FILENAME, edit_mode) as file:
        if put_header:
            file.write(HEADER)
        
        date = datetime.today().strftime('%d-%m-%Y')
        time = datetime.today().strftime('%H:%M')
        file.write(f"{filename};{date};{time}\n")


def get_download_date() -> str:
    FILENAME = f"{BASE_DIR_DOWNLOAD}/dowload_history.csv"
    return datetime.today().strftime('%d-%m-%Y')